/*
 * Copyright (C) 2009 SHARP Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.co.sharp.android.SampleIrdaIrss;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import jp.co.sharp.android.io.obex.client.PushClient;


/**
 * 赤外線送信のアクティビティである。
 * PushClient APIを使用して赤外線を使用したファイル送信を行う。
 * 送信前にコンテンツマネージャを呼び出し、コンテンツマネージャで選択したファイルを送信する。
 */
public class SampleIrdaIrssSendActivity extends Activity {
    ProgressDialog mProgressDialog = null;
    PushClient mPushClient = null;
    SampleFilePickerStatusModel mStatusModel;
    Uri mUri;

    /**
     *  アクティビティ作成時に呼ばれるメソッドである。
     *  ボタンのクリックリスナの設定を行う。
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.sample_irda_irss_send);

        Button btn = (Button)findViewById(R.id.send_start);
        btn.setOnClickListener(new OnClickListener() {
            public void onClick(View view) {
                /* インテント作成 */
                Intent intent = new Intent();
                /* アクション設定 */
                intent.setAction(Intent.ACTION_PICK);
                /* クラス設定 */
                intent.setClassName("jp.co.sharp.android.cm", "jp.co.sharp.android.cm.main.view.ContentManagerActivity");
                /* コンテンツマネージャ起動 */
                startActivityForResult(intent, 0);
            }
        });
    }

    /**
     * 進捗ダイアログの表示を行うメソッドである。
     */
    private void showProgress() {
        /* 進捗ダイアログ作成 */
        mProgressDialog = new ProgressDialog(this);
        /* タイトル設定 */
        mProgressDialog.setTitle(R.string.sample_irda_title);
        /* メッセージ設定 */
        mProgressDialog.setMessage(getResources().getString(R.string.sample_irda_wait));
        /* 戻るボタン無効 */
        mProgressDialog.setCancelable(false);
        /* バー型に設定 */
        mProgressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        /* ファイルサイズを進捗ダイアログに設定 */
        mProgressDialog.setMax((int)mStatusModel.getFileSize());
        /* ボタン追加 */
        mProgressDialog.setButton(getResources().getString(R.string.sample_irda_cancel),
                new DialogInterface.OnClickListener() {
            /**
             * ボタンが押されたときに呼ばれるメソッドである。
             */
            public void onClick(DialogInterface dialog, int whichButton) {
                if (mPushClient != null) {
                    mPushClient.cancel();
                    mPushClient = null;
                }
                mProgressDialog.dismiss();
            }
        });
        mProgressDialog.show();
    }

    @Override
    protected void onStop() {
        super.onStop();

        /* 通信中にアクティビティが停止した場合、送信を中断する。 */
        if (mPushClient != null) {
            mPushClient.cancel();
            mPushClient = null;
        }
        /* 通信中にアクティビティが停止した場合、進捗ダイアログを閉じる。 */
        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
        }
    }

    /**
     * アクティビティの戻り値を取得する。
     * 戻り値がRESULT_OKだった場合、送信を開始する。
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (resultCode == RESULT_OK) {
            sending(data);
        }
    }

    /**
     * 送信設定を行うメソッドである。
     * インテントフィルタに取得するアクションを設定し、ブロードキャストレシーバーに登録する。
     */
    public void sending(Intent retIntent) {
        /* コンテンツマネージャより取得したUriをファイル情報に変換する */
        SampleFilePickerGetContentStatus getContentsStatus = new SampleFilePickerGetContentStatus ();
        mUri = retIntent.getData();
        mStatusModel = getContentsStatus.getContentOne(mUri, this);
        /* インテントフィルタ作成 */
        IntentFilter filter = new IntentFilter();
        /* インテントフィルタ設定 */
        filter.addAction(PushClient.CONNECTION_STATUS);
        filter.addAction(PushClient.PUT_STARTED);
        filter.addAction(PushClient.PUT_PROGRESS);
        filter.addAction(PushClient.PUT_FINISHED);
        /* ブロードキャストレシーバに登録 */
        registerReceiver(mBroadcastReceiver, filter);
        /* 送信クラス作成 */
        mPushClient = new PushClient();
        Uri uri = Uri.parse("file://" + mStatusModel.getFilePath());
        /* 送信開始 */
        mPushClient.put(PushClient.TYPE_AUTO, null, mStatusModel.getFileNameFull(), null, uri);    /* ★Addon用API使用箇所★ */
        /* 進捗ダイアログ表示 */
        showProgress();
    }

    /**
     * ブロードキャストレシーバークラスである。
     * 登録したアクションがブロードキャストされたときにonReceiveが呼ばれる。
     * 接続状態、受信開始、受信中、受信終了を受け取る。
     */
    private BroadcastReceiver mBroadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (action.equals(PushClient.CONNECTION_STATUS)) {
                int status = intent.getIntExtra("status", PushClient.STATUS_NOT_CONNECTED);
                int result = intent.getIntExtra("result", PushClient.RESULT_ERROR_OTHER);
                if (status == PushClient.STATUS_NOT_CONNECTED) {
                    if (result == PushClient.RESULT_SUCCESS) {
                        /* 送信が終了し、切断完了です */
                    } else {
                        /* エラー終了です */
                    }
                    mProgressDialog.dismiss();
                    mPushClient = null;
                } else {
                    /* 接続完了です */
                }
            } else if (action.equals(PushClient.PUT_STARTED)) {
                /* オブジェクトの送信を開始します。 */
            } else if (action.equals(PushClient.PUT_PROGRESS)) {
                /* 送信処理中。送信済みサイズを取得可能です。 */
                int sent = intent.getIntExtra("sent", PushClient.RESULT_ERROR_OTHER);
                mProgressDialog.setProgress(sent);
            } else if (action.equals(PushClient.PUT_FINISHED)) {
                /* オブジェクトの送信終了です */
                /* 複数オブジェクトを送信している場合は、1件ごとにこのIntentを受信します。 */
                int result = intent.getIntExtra("result",PushClient.RESULT_ERROR_OTHER);
                if (result == PushClient.RESULT_SUCCESS) {
                    /* 送信中のオブジェクトの送信に成功した */
                    MessageDialog messageDialog = new MessageDialog();
                    messageDialog.show(context, R.string.sample_irda_send_success);
                } else {
                    /* 送信中のオブジェクトの送信に失敗した */
                    MessageDialog messageDialog = new MessageDialog();
                    messageDialog.show(context, R.string.sample_irda_send_failed);
                }
            }
        }
    };

}

